"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformImg = void 0;
const picgo_1 = require("picgo");
const fs_extra_1 = __importDefault(require("fs-extra"));
const log_1 = require("../log");
const path_1 = require("path");
const helpUpload = (imgConfig, imgs) => __awaiter(void 0, void 0, void 0, function* () {
    const picgo = new picgo_1.PicGo(imgConfig);
    return new Promise((resolve, reject) => __awaiter(void 0, void 0, void 0, function* () {
        picgo.helper.beforeUploadPlugins.register('rename', {
            handle: function (ctx) {
                ctx.output.forEach((x) => {
                    log_1.debug('fileName', x.fileName);
                    const ext = path_1.extname(x.fileName);
                    const name = path_1.basename(x.fileName, ext);
                    return (x.fileName = `${name}_${Date.now()}${ext}`);
                });
            },
        });
        picgo
            .on('uploadProgress', (progress) => {
            log_1.info('上传进度:', progress);
        })
            .on('failed', (e) => {
            log_1.error('上传失败 failed', JSON.stringify(e));
            reject();
        })
            .on('notification', (notice) => {
            log_1.error('上传失败 notification', JSON.stringify(notice));
        });
        const map = {};
        const output = (yield picgo.upload(imgs));
        output.forEach((img, i) => {
            map[imgs[i]] = img.imgUrl;
        });
        resolve(map);
    }));
});
const transformImg = function (files) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const imgConfig = this.config.imgConfig;
        if (!imgConfig) {
            return;
        }
        log_1.info('转存图片开始');
        const map = new Map();
        // 获取图片地址
        for (let file of files) {
            const content = fs_extra_1.default.readFileSync(file, 'utf8');
            const urls = (_a = content.match(/\!\[.*\]\(.*\)/g)) === null || _a === void 0 ? void 0 : _a.map((i) => {
                const url = i.match(/\!\[.*\]\((.*?)( ".*")?\)/)[1];
                return url;
            });
            if (urls) {
                map.set(file, urls);
            }
        }
        const upload = [];
        for (let l of map.values()) {
            upload.push(...l);
        }
        const originNewMap = Object.create(null);
        log_1.debug('上传列表', upload.join('\n'));
        // 每次传5张。串行传
        log_1.info(`开始转存图片，共${upload.length}张`);
        let errorInfo = [];
        let length = 5;
        for (let i = 0; i < upload.length; i += length) {
            try {
                log_1.info(`转存中: ${i}/${upload.length}...`);
                log_1.info(upload.slice(i, i + length).join('\n'));
                const result = yield helpUpload(this.config.imgConfig, upload.slice(i, i + length));
                Object.assign(originNewMap, result);
            }
            catch (e) {
                // error(e)
                log_1.error('转存错误', e);
                errorInfo.push(...upload.slice(i, i + length));
            }
        }
        // 替换图片
        for (let file of files) {
            let content = fs_extra_1.default.readFileSync(file, 'utf8');
            const urls = (_b = content.match(/\!\[.*\]\(.*\)/g)) === null || _b === void 0 ? void 0 : _b.map((i) => {
                const url = i.match(/\!\[.*\]\((.*?)( ".*")?\)/)[1];
                return url;
            });
            if (urls) {
                urls.forEach((url) => {
                    if (originNewMap[url]) {
                        content = content.replace(new RegExp(url, 'g'), originNewMap[url]);
                    }
                });
                fs_extra_1.default.writeFileSync(file, content);
            }
        }
        log_1.warn('以下图片转存失败，请手动确认\n', errorInfo.join('\n'));
    });
};
exports.transformImg = transformImg;
//# sourceMappingURL=transformImg.js.map